# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2018 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Handles complete circuits made of parts and nets.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import range
from builtins import str
from future import standard_library
standard_library.install_aliases()

try:
    import __builtin__ as builtins
except ImportError:
    import builtins

import os.path
import inspect
import time
import graphviz


from .pckg_info import __version__
from .utilities import *


class Circuit(object):
    """
    Class object that holds the entire netlist of parts and nets.

    Attributes:
        parts: List of all the schematic parts as Part objects.
        nets: List of all the schematic nets as Net objects.
        buses: List of all the buses as Bus objects.
        hierarchy: A '.'-separated concatenation of the names of nested
            SubCircuits at the current time it is read.
        level: The current level in the schematic hierarchy.
        context: Stack of contexts for each level in the hierarchy.
    """

    OK, WARNING, ERROR = range(3)

    def __init__(self, **kwargs):
        """Initialize the Circuit object."""
        self.reset(init=True)

        # Set passed-in attributes for the circuit.
        for k, v in kwargs.items():
            setattr(self, k, v)

    def reset(self, init=False):
        """Clear any circuitry and cached part libraries and start over."""

        from .SchLib import SchLib

        # Clear circuitry.
        self.mini_reset(init)

        # Also clear any cached libraries.
        SchLib.reset()
        global backup_lib
        backup_lib = None

    def mini_reset(self, init=False):
        """Clear any circuitry but don't erase any loaded part libraries."""

        from .Net import NCNet

        self.name = ''
        self.parts = []
        self.nets = []
        self.buses = []
        self.hierarchy = 'top'
        self.level = 0
        self.context = [('top', )]

        # Clear out the no-connect net and set the global no-connect if it's
        # tied to this circuit.
        self.NC = NCNet(
            name='__NOCONNECT', circuit=self
        )  # Net for storing no-connects for parts in this circuit.
        if not init and self is default_circuit:
            builtins.NC = self.NC

    def add_parts(self, *parts):
        """Add some Part objects to the circuit."""
        for part in parts:
            # Add the part to this circuit if the part is movable and
            # it's not already in this circuit.
            if part.circuit != self:
                if part.is_movable():

                    # Remove the part from the circuit it's already in.
                    if isinstance(part.circuit, Circuit):
                        part.circuit -= part

                    # Add the part to this circuit.
                    part.circuit = self  # Record the Circuit object the part belongs to.
                    part.ref = part.ref  # This adjusts the part reference if necessary.
                    part.hierarchy = self.hierarchy  # Tag the part with its hierarchy position.

                    # To determine where this part was created, trace the function
                    # calls that led to this part and place into a field
                    # but strip off all the calls to internal SKiDL functions.
                    call_stack = inspect.stack()  # Get function call stack.
                    # Use the function at the top of the stack to
                    # determine the location of the SKiDL library functions.
                    try:
                        skidl_dir, _ = os.path.split(call_stack[0].filename)
                    except AttributeError:
                        skidl_dir, _ = os.path.split(call_stack[0][1])
                    # Record file_name#line_num starting from the bottom of the stack
                    # and terminate as soon as a function is found that's in the
                    # SKiDL library (no use recording internal calls).
                    skidl_trace = []
                    for frame in reversed(call_stack):
                        try:
                            filename = frame.filename
                            lineno = frame.lineno
                        except AttributeError:
                            filename = frame[1]
                            lineno = frame[2]
                        if os.path.split(filename)[0] == skidl_dir:
                            # Found function in SKiDL library, so trace is complete.
                            break
                        # Get the absolute path to the file containing the function
                        # and the line number of the call in the file. Append these
                        # to the trace.
                        filepath = os.path.abspath(filename)
                        skidl_trace.append('#'.join((filepath, str(lineno))))
                    # Store the function call trace into a part field.
                    if skidl_trace:
                        part.skidl_trace = ';'.join(skidl_trace)

                    self.parts.append(part)

                else:
                    logger.error(
                        "Can't add unmovable part {} to this circuit.".format(
                            part.ref))
                    raise Exception

    def rmv_parts(self, *parts):
        """Remove some Part objects from the circuit."""
        for part in parts:
            if part.is_movable():
                if part.circuit == self and part in self.parts:
                    part.circuit = None
                    part.hierarchy = None
                    self.parts.remove(part)
                else:
                    logger.warning(
                        "Removing non-existent part {} from this circuit.".
                        format(part.ref))
            else:
                logger.error("Can't remove part {} from this circuit.".format(
                    part.ref))
                raise Exception

    def add_nets(self, *nets):
        """Add some Net objects to the circuit. Assign a net name if necessary."""
        for net in nets:
            # Add the net to this circuit if the net is movable and
            # it's not already in this circuit.
            if net.circuit != self:
                if net.is_movable():

                    # Remove the net from the circuit it's already in.
                    if isinstance(net.circuit, Circuit):
                        net.circuit -= net

                    # Add the net to this circuit.
                    net.circuit = self  # Record the Circuit object the net belongs to.
                    net.name = net.name
                    net.hierarchy = self.hierarchy  # Tag the net with its hierarchy position.
                    self.nets.append(net)

                else:
                    logger.error(
                        "Can't add unmovable net {} to this circuit.".format(
                            net.name))
                    raise Exception

    def rmv_nets(self, *nets):
        """Remove some Net objects from the circuit."""
        for net in nets:
            if net.is_movable():
                if net.circuit == self and net in self.nets:
                    net.circuit = None
                    net.hierarchy = None
                    self.nets.remove(net)
                else:
                    logger.warning(
                        "Removing non-existent net {} from this circuit.".
                        format(net.name))
            else:
                logger.error(
                    "Can't remove unmovable net {} from this circuit.".format(
                        net.name))
                raise Exception

    def add_buses(self, *buses):
        """Add some Bus objects to the circuit. Assign a bus name if necessary."""
        for bus in buses:
            # Add the bus to this circuit if the bus is movable and
            # it's not already in this circuit.
            if bus.circuit != self:
                if bus.is_movable():

                    # Remove the bus from the circuit it's already in, but skip
                    # this if the bus isn't already in a Circuit.
                    if isinstance(bus.circuit, Circuit):
                        bus.circuit -= bus

                    # Add the bus to this circuit.
                    bus.circuit = self
                    bus.name = bus.name
                    bus.hierarchy = self.hierarchy  # Tag the bus with its hierarchy position.
                    self.buses.append(bus)
                    for net in bus.nets:
                        self += net

    def rmv_buses(self, *buses):
        """Remove some buses from the circuit."""
        for bus in buses:
            if bus.is_movable():
                if bus.circuit == self and bus in self.buses:
                    bus.circuit = None
                    bus.hierarchy = None
                    self.buses.remove(bus)
                    for net in bus.nets:
                        self.nets.remove(net)
                else:
                    logger.warning(
                        "Removing non-existent bus {} from this circuit.".
                        format(bus.name))
            else:
                logger.error(
                    "Can't remove unmovable bus {} from this circuit.".format(
                        bus.name))
                raise Exception

    def add_parts_nets_buses(self, *parts_nets_buses):
        """Add Parts, Nets and Buses to the circuit."""

        from .Part import Part
        from .Net import Net
        from .Bus import Bus

        for pnb in flatten(parts_nets_buses):
            if isinstance(pnb, Part):
                self.add_parts(pnb)
            elif isinstance(pnb, Net):
                self.add_nets(pnb)
            elif isinstance(pnb, Bus):
                self.add_buses(pnb)
            else:
                logger.error("Can't add a {} to a Circuit object.".format(
                    type(pnb)))
                raise Exception
        return self

    def rmv_parts_nets_buses(self, *parts_nets_buses):
        """Remove Parts, Nets and Buses from the circuit."""

        from .Net import Net
        from .Bus import Bus
        from .Part import Part

        for pnb in flatten(parts_nets_buses):
            if isinstance(pnb, Part):
                self.rmv_parts(pnb)
            elif isinstance(pnb, Net):
                self.rmv_nets(pnb)
            elif isinstance(pnb, Bus):
                self.rmv_buses(pnb)
            else:
                logger.error("Can't remove a {} from a Circuit object.".format(
                    type(pnb)))
                raise Exception
        return self

    __iadd__ = add_parts_nets_buses
    __isub__ = rmv_parts_nets_buses

    def get_nets(self):
        """Get all the distinct nets for the circuit."""
        distinct_nets = []
        for net in self.nets:
            if net is self.NC:
                # Exclude no-connect net.
                continue
            if not net.get_pins():
                # Exclude empty nets with no attached pins.
                continue
            for n in distinct_nets:
                # Exclude net if its already attached to a previously selected net.
                if net.is_attached(n):
                    break
            else:
                # This net is not attached to any of the other distinct nets,
                # so it is also distinct.
                distinct_nets.append(net)
        return distinct_nets

    def _erc_setup(self):
        """
        Initialize the electrical rules checker.
        """

        from skidl import Pin

        # Initialize the pin contention matrix.
        self._erc_matrix = [[self.OK for c in range(11)] for r in range(11)]
        self._erc_matrix[Pin.OUTPUT][Pin.OUTPUT] = self.ERROR
        self._erc_matrix[Pin.TRISTATE][Pin.OUTPUT] = self.WARNING
        self._erc_matrix[Pin.UNSPEC][Pin.INPUT] = self.WARNING
        self._erc_matrix[Pin.UNSPEC][Pin.OUTPUT] = self.WARNING
        self._erc_matrix[Pin.UNSPEC][Pin.BIDIR] = self.WARNING
        self._erc_matrix[Pin.UNSPEC][Pin.TRISTATE] = self.WARNING
        self._erc_matrix[Pin.UNSPEC][Pin.PASSIVE] = self.WARNING
        self._erc_matrix[Pin.UNSPEC][Pin.UNSPEC] = self.WARNING
        self._erc_matrix[Pin.PWRIN][Pin.TRISTATE] = self.WARNING
        self._erc_matrix[Pin.PWRIN][Pin.UNSPEC] = self.WARNING
        self._erc_matrix[Pin.PWROUT][Pin.OUTPUT] = self.ERROR
        self._erc_matrix[Pin.PWROUT][Pin.BIDIR] = self.WARNING
        self._erc_matrix[Pin.PWROUT][Pin.TRISTATE] = self.ERROR
        self._erc_matrix[Pin.PWROUT][Pin.UNSPEC] = self.WARNING
        self._erc_matrix[Pin.PWROUT][Pin.PWROUT] = self.ERROR
        self._erc_matrix[Pin.OPENCOLL][Pin.OUTPUT] = self.ERROR
        self._erc_matrix[Pin.OPENCOLL][Pin.TRISTATE] = self.ERROR
        self._erc_matrix[Pin.OPENCOLL][Pin.UNSPEC] = self.WARNING
        self._erc_matrix[Pin.OPENCOLL][Pin.PWROUT] = self.ERROR
        self._erc_matrix[Pin.OPENEMIT][Pin.OUTPUT] = self.ERROR
        self._erc_matrix[Pin.OPENEMIT][Pin.BIDIR] = self.WARNING
        self._erc_matrix[Pin.OPENEMIT][Pin.TRISTATE] = self.WARNING
        self._erc_matrix[Pin.OPENEMIT][Pin.UNSPEC] = self.WARNING
        self._erc_matrix[Pin.OPENEMIT][Pin.PWROUT] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.INPUT] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.OUTPUT] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.BIDIR] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.TRISTATE] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.PASSIVE] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.UNSPEC] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.PWRIN] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.PWROUT] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.OPENCOLL] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.OPENEMIT] = self.ERROR
        self._erc_matrix[Pin.NOCONNECT][Pin.NOCONNECT] = self.ERROR

        # Fill-in the other half of the symmetrical matrix.
        for c in range(1, 11):
            for r in range(c):
                self._erc_matrix[r][c] = self._erc_matrix[c][r]

    def set_pin_conflict_rule(self, pin1_func, pin2_func, conflict_level):
        """
        Set the level of conflict for two types of pins on the same net.

        Args:
            pin1_func: The function of the first pin (e.g., Pin.OUTPUT).
            pin2_func: The function of the second pin (e.g., Pin.TRISTATE).
            conflict_level: Severity of conflict (e.g., self.OK, self.WARNING, self.ERROR).
        """

        # Place the conflict level into the symmetrical ERC matrix.
        self._erc_matrix[pin1_func][pin2_func] = conflict_level
        self._erc_matrix[pin2_func][pin1_func] = conflict_level

    def erc_pin_to_pin_chk(self, pin1, pin2):
        """Check for conflict between two pins on a net."""

        # Use the functions of the two pins to index into the ERC table
        # and see if the pins are compatible (e.g., an input and an output)
        # or incompatible (e.g., a conflict because both are outputs).
        return self._erc_matrix[pin1.func][pin2.func]

    def ERC(self):
        """
        Do an electrical rules check on the circuit.
        """

        self._erc_setup()

        # Check the nets for errors.
        for net in self.nets:
            net.erc()

        # Check the parts for errors.
        for part in self.parts:
            part.erc()

        if (erc_logger.error.count, erc_logger.warning.count) == (0, 0):
            sys.stderr.write('\nNo ERC errors or warnings found.\n\n')
        else:
            sys.stderr.write('\n{} warnings found during ERC.\n'.format(
                erc_logger.warning.count))
            sys.stderr.write('{} errors found during ERC.\n\n'.format(
                erc_logger.error.count))

    def generate_netlist(self, **kwargs):
        """
        Return a netlist and also write it to a file/stream.

        Args:
            file_: Either a file object that can be written to, or a string
                containing a file name, or None.
            tool: The EDA tool the netlist will be generated for.
            do_backup: If true, create a library with all the parts in the circuit.

        Returns:
            A netlist.
        """

        import skidl

        # Extract arguments:
        #     Get EDA tool the netlist will be generated for.
        #     Get file the netlist will be stored in (if any).
        #     Get flag controlling the generation of a backup library.
        tool = kwargs.pop('tool', skidl.get_default_tool())
        file_ = kwargs.pop('file_', None)
        do_backup = kwargs.pop('do_backup', True)

        try:
            gen_func = getattr(self, '_gen_netlist_{}'.format(tool))
            netlist = gen_func(**kwargs)  # Pass any remaining arguments.
        except KeyError:
            logger.error(
                "Can't generate netlist in an unknown ECAD tool format ({}).".
                format(tool))
            raise Exception

        if (logger.error.count, logger.warning.count) == (0, 0):
            sys.stderr.write(
                '\nNo errors or warnings found during netlist generation.\n\n')
        else:
            sys.stderr.write(
                '\n{} warnings found during netlist generation.\n'.format(
                    logger.warning.count))
            sys.stderr.write(
                '{} errors found during netlist generation.\n\n'.format(
                    logger.error.count))

        with opened(file_ or (get_script_name() + '.net'), 'w') as f:
            f.write(str(netlist))

        if do_backup:
            self.backup_parts(
            )  # Create a new backup lib for the circuit parts.
            global backup_lib  # Clear out any old backup lib so the new one
            backup_lib = None  #   will get reloaded when it's needed.

        return netlist

    def generate_xml(self, file_=None, tool=None):
        """
        Return netlist as an XML string and also write it to a file/stream.

        Args:
            file_: Either a file object that can be written to, or a string
                containing a file name, or None.

        Returns:
            A string containing the netlist.
        """

        import skidl

        if tool is None:
            tool = skidl.get_default_tool()

        try:
            gen_func = getattr(self, '_gen_xml_{}'.format(tool))
            netlist = gen_func()
        except KeyError:
            logger.error(
                "Can't generate XML in an unknown ECAD tool format ({}).".
                format(tool))
            raise Exception

        if (logger.error.count, logger.warning.count) == (0, 0):
            sys.stderr.write(
                '\nNo errors or warnings found during XML generation.\n\n')
        else:
            sys.stderr.write(
                '\n{} warnings found during XML generation.\n'.format(
                    logger.warning.count))
            sys.stderr.write(
                '{} errors found during XML generation.\n\n'.format(
                    logger.error.count))

        with opened(file_ or (get_script_name() + '.xml'), 'w') as f:
            f.write(netlist)

        return netlist

    def generate_graph(self,
                       file_=None,
                       engine='neato',
                       rankdir='LR',
                       part_shape='rectangle',
                       net_shape='point',
                       splines=None,
                       show_values=True,
                       show_anon=False):
        """
        Returns a graphviz graph as graphviz object and can also write it to a file/stream.
        When used in ipython the graphviz object will drawn as an SVG in the output.

        See https://graphviz.readthedocs.io/en/stable/ and http://graphviz.org/doc/info/attrs.html

        Args:
            file_: A string containing a file name, or None.
            engine: See graphviz documentation
            rankdir: See graphviz documentation
            part_shape: Shape of the part nodes
            net_shape: Shape of the net nodes
            splines: Style for the edges, try 'ortho' for a schematic like feel
            show_values: Show values as external labels on part nodes
            show_anon: Show anonymous net names

        Returns:
            graphviz.Digraph
        """
        dot = graphviz.Digraph(engine=engine)
        dot.attr(rankdir=rankdir, splines=splines)

        nets = self.get_nets()

        # try and keep things in the same order
        nets.sort(key=lambda n: n.name.lower())

        for n in nets:
            xlabel = n.name
            if not show_anon and n.is_implicit():
                xlabel = None
            dot.node(n.name, shape=net_shape, xlabel=xlabel)
            for pin in n.pins:
                dot.edge(pin.part.ref, n.name, arrowhead='none')

        for p in sorted(self.parts, key=lambda p: p.ref.lower()):
            xlabel = None
            if show_values:
                xlabel = p.value
            dot.node(p.ref, shape=part_shape, xlabel=xlabel)

        if file_ is not None:
            dot.save(file_)
        return dot

    def backup_parts(self, file_=None):
        """
        Saves parts in circuit as a SKiDL library in a file.

        Args:
            file: Either a file object that can be written to, or a string
                containing a file name, or None. If None, a standard library
                file will be used.

        Returns:
            Nothing.
        """

        import skidl
        from .defines import SKIDL

        lib = skidl.SchLib(tool=SKIDL)  # Create empty library.
        for p in self.parts:
            lib += p
        if not file_:
            file_ = skidl.BACKUP_LIB_FILE_NAME
        lib.export(libname=skidl.BACKUP_LIB_NAME, file_=file_)


def SubCircuit(f):
    """
    A @SubCircuit decorator is used to create hierarchical circuits.

    Args:
        f: The function containing SKiDL statements that represents a subcircuit.
    """

    def sub_f(*args, **kwargs):
        # Upon entry, save the reference to the default Circuit object.
        save_default_circuit = default_circuit  # pylint: disable=undefined-variable

        # If the subcircuit has no 'circuit' argument, then all the SKiDL
        # statements in the subcircuit function will reference the default Circuit
        # object.
        if 'circuit' not in kwargs:
            circuit = default_circuit  # pylint: disable=undefined-variable

        # But if the subcircuit function has a 'circuit' argument, then set the default
        # Circuit object to that. Then all SKiDL statements in the function will
        # make changes (i.e., add parts, nets, buses) to that.
        else:
            circuit = kwargs['circuit']
            del kwargs[
                'circuit']  # Don't pass the circuit parameter down to the f function.
            builtins.default_circuit = circuit

        # Setup some globals needed in the subcircuit.
        builtins.NC = default_circuit.NC  # pylint: disable=undefined-variable

        # Invoking the subcircuit function creates circuitry at a level one
        # greater than the current level. (The top level is zero.)
        circuit.level += 1

        # Create a name for this subcircuit from the concatenated names of all
        # the nested subcircuit functions that were called on all the preceding levels
        # that led to this one.
        circuit.hierarchy = circuit.context[-1][0] + '.' + f.__name__

        # Store the context so it can be used if this subcircuit function
        # invokes another subcircuit function within itself to add more
        # levels of hierarchy.
        circuit.context.append((circuit.hierarchy, ))

        # Call the function to create whatever circuitry it handles.
        # The arguments to the function are usually nets to be connected to the
        # parts instantiated in the function, but they may also be user-specific
        # and have no effect on the mechanics of adding parts or nets although
        # they may direct the function as to what parts and nets get created.
        # Store any results it returns as a list. These results are user-specific
        # and have no effect on the mechanics of adding parts or nets.
        results = f(*args, **kwargs)

        # Restore the context that existed before the subcircuitry was
        # created. This does not remove the circuitry since it has already been
        # added to the parts and nets lists.
        circuit.context.pop()

        # Restore the hierarchy label and level.
        circuit.hierarchy = circuit.context[-1][0]
        circuit.level -= 1

        # Restore the default circuit and globals.
        builtins.default_circuit = save_default_circuit
        builtins.NC = default_circuit.NC  # pylint: disable=undefined-variable

        return results

    return sub_f


# The decorator can also be called as "@subcircuit".
subcircuit = SubCircuit



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

silabs = SchLib(tool=SKIDL).add_parts(*[
        Part(name='CP2102N-A01-GQFN24',dest=TEMPLATE,tool=SKIDL,keywords='USB UART bridge',description='USB to UART master bridge',ref_prefix='U',num_units=1,fplist=['QFN*4x4mm*Pitch0.5mm*'],do_erc=True,pins=[
            Pin(num='1',name='RI/CLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='D+',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='D-',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='VIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='REGIN',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VBUS',do_erc=True),
            Pin(num='9',name='~RSTb',do_erc=True),
            Pin(num='10',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='20',name='RXD',do_erc=True),
            Pin(num='11',name='GPIO.3',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='GPIO.2',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='DSR',do_erc=True),
            Pin(num='13',name='GPIO.1',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='DTR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='GPIO.0',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='DCD',do_erc=True),
            Pin(num='15',name='~SUSPENDb',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='17',name='SUSPEND',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='CTS',do_erc=True),
            Pin(num='19',name='RTS',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='CP2104',dest=TEMPLATE,tool=SKIDL,keywords='uart usb bridge interface transceiver',description='Single-Chip USB-to-UART Bridge, USB 2.0 Full-Speed, 2Mbps UART, QFN package',ref_prefix='U',num_units=1,fplist=['QFN*4x4mm*Pitch0.5mm*'],do_erc=True,pins=[
            Pin(num='1',name='RI',do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='D+',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='D-',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='VIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='REGIN',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VBUS',do_erc=True),
            Pin(num='9',name='~RST',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='VIO/NC',func=Pin.PWROUT,do_erc=True),
            Pin(num='20',name='RXD',do_erc=True),
            Pin(num='11',name='GPIO.3',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='GPIO.2',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='DSR',do_erc=True),
            Pin(num='13',name='GPIO.1',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='DTR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='GPIO.0',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='DCD',do_erc=True),
            Pin(num='15',name='~SUSPEND',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='PAD',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VPP',func=Pin.PASSIVE,do_erc=True),
            Pin(num='17',name='SUSPEND',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='CTS',do_erc=True),
            Pin(num='19',name='RTS',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='CP2112',dest=TEMPLATE,tool=SKIDL,keywords='USB SMBUS bridge',description='HID USB to SMBUS master bridge',ref_prefix='U',num_units=1,fplist=['QFN*4x4mm*Pitch0.5mm*'],do_erc=True,pins=[
            Pin(num='1',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='D+',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='D-',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='VIO',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='REGIN',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VBUS',do_erc=True),
            Pin(num='9',name='~RST',do_erc=True),
            Pin(num='10',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='20',name='GPIO.3',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='SUSPEND',func=Pin.OUTPUT,do_erc=True),
            Pin(num='21',name='GPIO.2',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='GPIO.7',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='GPIO.1',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='GPIO.6',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='GPIO.0',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='GPIO.5',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='SCL',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='GPIO.4',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VPP',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='~SUSPEND',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='19',name='NC',func=Pin.NOCONNECT,do_erc=True)]),
        Part(name='Si4362',dest=TEMPLATE,tool=SKIDL,keywords='radio receiver',description='EZRadioPRO Low Current Receiver',ref_prefix='U',num_units=1,fplist=['QFN*4x4mm*Pitch0.5mm*'],do_erc=True,pins=[
            Pin(num='1',name='SDN',do_erc=True),
            Pin(num='2',name='RXp',do_erc=True),
            Pin(num='3',name='RXn',do_erc=True),
            Pin(num='4',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='8',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='GPIO0',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='GPIO1',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='GPIO3',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='~IRQ',func=Pin.OUTPUT,do_erc=True),
            Pin(num='21',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='SCLK',do_erc=True),
            Pin(num='13',name='SDO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='SDI',do_erc=True),
            Pin(num='15',name='~SEL',do_erc=True),
            Pin(num='16',name='XOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='17',name='XIN',do_erc=True),
            Pin(num='18',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='GPIO2',func=Pin.BIDIR,do_erc=True)]),
        Part(name='Si4735-D60-GU',dest=TEMPLATE,tool=SKIDL,keywords='Broadcast AM FM SW LW Radio Receiver RDS RBDS',description='AM/FM/SW/LW Broadcast Radio Receiver with RDS/RBDS',ref_prefix='U',num_units=1,fplist=['SSOP-24_3.9x8.7mm_Pitch0.635mm*'],do_erc=True,aliases=['Si4734-D60-GU', 'Si4731-D60-GU', 'Si4730-D60-GU'],pins=[
            Pin(num='1',name='DOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='DFS',do_erc=True),
            Pin(num='3',name='GPO3/[DCLK]',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='GPO2/[~INT~]',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='GPO1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='7',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='8',name='FMI',do_erc=True),
            Pin(num='9',name='RFGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='20',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='21',name='VA',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='AMI',do_erc=True),
            Pin(num='22',name='DBYP',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='23',name='ROUT/[DOUT]',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='LOUT/[DFS]',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='~RST',do_erc=True),
            Pin(num='16',name='~SEN',do_erc=True),
            Pin(num='17',name='SCLK',do_erc=True),
            Pin(num='18',name='SDIO',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='RCLK',do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

ac_dc = SchLib(tool=SKIDL).add_parts(*[
        Part(name='FSDH321',dest=TEMPLATE,tool=SKIDL,keywords='SMPS Controller with MOSFET 17W AC-DC',description='17W SMPS Controller, 50kHz, AC-DC, PDIP-8',ref_prefix='U',num_units=1,fplist=['DIP*', 'PDIP*'],do_erc=True,aliases=['FSDL321'],pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='FB',do_erc=True),
            Pin(num='4',name='IPK',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='VSTR',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='D',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FSDH321L',dest=TEMPLATE,tool=SKIDL,keywords='SMPS Controller with MOSFET 17W AC-DC',description='17W SMPS Controller, 50kHz, AC-DC, SMD-8',ref_prefix='U',num_units=1,fplist=['SMD*'],do_erc=True,aliases=['FSDL321L'],pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='FB',do_erc=True),
            Pin(num='4',name='IPK',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='VSTR',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='D',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FSL136MRT',dest=TEMPLATE,tool=SKIDL,keywords='SMPS Controller 50W AC-DC',description='67kHz SMPS Controller w/ Soft Start, max. 50W AC-DC, TO-220F-6L',ref_prefix='U',num_units=1,fplist=['TO-220-6L'],do_erc=True,pins=[
            Pin(num='1',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='FB',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='VST',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FSQ0565RSWDTU',dest=TEMPLATE,tool=SKIDL,keywords='Quasi Resonant SMPS Controller 80W AC-DC',description='67kHz Quasi Resonant SMPS Controller w/ Soft Start, max. 80W AC-DC, TO-220F-6L',ref_prefix='U',num_units=1,fplist=['TO-220-6L'],do_erc=True,aliases=['FSQ0565RQWDTU', 'FSQ0565RSLDTU', 'FSQ0565RQLDTU'],pins=[
            Pin(num='1',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='FB',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='SYNC',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='VST',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='HS-40003',dest=TEMPLATE,tool=SKIDL,keywords='24V 3W AC-DC module power supply',description='24V, 3W, AC-DC module power supply, Hahn',ref_prefix='U',num_units=1,fplist=['ACDC*Hahn*HS*400XX*'],do_erc=True,aliases=['HS-40005', 'HS-40009', 'HS-40012', 'HS-40015', 'HS-40018', 'HS-40024'],pins=[
            Pin(num='1',name='AC/N',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='AC/L',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='+Vout',func=Pin.PWROUT,do_erc=True),
            Pin(num='9',name='-Vout',func=Pin.PWROUT,do_erc=True)]),
        Part(name='IRM-02-3.3',dest=TEMPLATE,tool=SKIDL,keywords='9V 2W miniature AC-DC module-type power supply',description='9V 2W miniature AC-DC module-type power supply MeanWell',ref_prefix='U',num_units=1,fplist=['ACDC?Converter?MeanWell?IRM?02?x*'],do_erc=True,aliases=['IRM-02-5', 'IRM-02-9', 'IRM-02-12', 'IRM-02-15', 'IRM-02-24'],pins=[
            Pin(num='1',name='AC/N',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='AC/L',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='-Vout',func=Pin.PWROUT,do_erc=True),
            Pin(num='4',name='+Vout',func=Pin.PWROUT,do_erc=True)]),
        Part(name='IRM-03-3.3S',dest=TEMPLATE,tool=SKIDL,keywords='5V 3W AC-DC module power supply',description='5V, 3W, AC-DC module power supply, MeanWell',ref_prefix='U',num_units=1,fplist=['ACDC*MeanWell*IRM*03*'],do_erc=True,aliases=['IRM-03-5S', 'IRM-03-12S', 'IRM-03-15S', 'IRM-03-24S'],pins=[
            Pin(num='1',name='AC/L',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='AC/N',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='10',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='11',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='12',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='22',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='13',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='14',name='-Vout',func=Pin.PWROUT,do_erc=True),
            Pin(num='24',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='15',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='16',name='+Vout',func=Pin.PWROUT,do_erc=True),
            Pin(num='26',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='17',name='NC',func=Pin.NOCONNECT,do_erc=True)]),
        Part(name='KA5H02659RN',dest=TEMPLATE,tool=SKIDL,keywords='SMPS Controller AC-DC',description='67kHz SMPS Controller, AC-DC, PDIP-8',ref_prefix='U',num_units=1,fplist=['DIP8*'],do_erc=True,aliases=['KA5M02659RN'],pins=[
            Pin(num='1',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='FB',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='D',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='KA5H0265RCTU',dest=TEMPLATE,tool=SKIDL,keywords='SMPS Controller AC-DC',description='KA5H0265RCTU, 100kHz SMPS Controller w/ Soft Start, AC-DC, TO-220F-5L(Forming)',ref_prefix='U',num_units=1,fplist=['TO-220-5L'],do_erc=True,aliases=['KA5H0265RCYDTU'],pins=[
            Pin(num='1',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='FB',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='SS',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='KA5M0265RTU',dest=TEMPLATE,tool=SKIDL,keywords='SMPS Controller AC-DC',description='KA5M0280RTDTU, 67kHz SMPS Controller, AC-DC, TO-220F-4L(Forming)',ref_prefix='U',num_units=1,fplist=['TO-220F-4L'],do_erc=True,aliases=['KA5M0265RYDTU', 'KA5L0265RTU', 'KA5L0265RYDTU', 'KA5H0280RTU', 'KA5H0280RYDTU', 'KA5M0280RTU', 'KA5M0280RYDTU'],pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='FB',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='NCP1200P',dest=TEMPLATE,tool=SKIDL,keywords='SMPS Controller AC-DC',description='SMPS Controller, AC-DC, PDIP-8',ref_prefix='U',num_units=1,fplist=['DIP8*', 'SOIC8*'],do_erc=True,aliases=['NCP1200D'],pins=[
            Pin(num='1',name='ADJ',do_erc=True),
            Pin(num='2',name='FB',do_erc=True),
            Pin(num='3',name='CS1',do_erc=True),
            Pin(num='4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='DRV',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='HV',do_erc=True)]),
        Part(name='UC3846X-D08-T',dest=TEMPLATE,tool=SKIDL,keywords='SMPS PWM Controller',description='UC3846x-DO8-T, SMPS Flyback PWM Controller, DIP8',ref_prefix='U',num_units=1,fplist=['DIP*', 'PDIP*'],do_erc=True,pins=[
            Pin(num='1',name='GATE',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='SENSE',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='RI',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='FB',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='VIPer22ADIP-E',dest=TEMPLATE,tool=SKIDL,keywords='SMPS',description='Low power OFF-line SMPS primary switcher',ref_prefix='U',num_units=1,fplist=['DIP8*', 'SO-8*'],do_erc=True,aliases=['VIPer22ASTR-E', 'VIPer22AS-E'],pins=[
            Pin(num='1',name='S',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='S',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='FB',do_erc=True),
            Pin(num='4',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='D',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='VIPer25LN',dest=TEMPLATE,tool=SKIDL,keywords='SMPS Controller with MOSFET 12W AC-DC',description='12W SMPS Controller, AC-DC, PDIP-7',ref_prefix='U',num_units=1,do_erc=True,aliases=['VIPer25HN'],pins=[
            Pin(num='1',name='S',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='ZCD',do_erc=True),
            Pin(num='4',name='FB',do_erc=True),
            Pin(num='5',name='BR',do_erc=True),
            Pin(num='7',name='D',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='D',func=Pin.PASSIVE,do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

motors = SchLib(tool=SKIDL).add_parts(*[
        Part(name='Fan',dest=TEMPLATE,tool=SKIDL,keywords='Fan Motor',description='Fan',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x02', 'Connect:bornier2', 'TerminalBlock*2pol'],do_erc=True,pins=[
            Pin(num='1',name='+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Fan_ALT',dest=TEMPLATE,tool=SKIDL,keywords='Fan Motor',description='Fan without PWM or tach, alternative symbol',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x02', 'Connect:bornier2', 'TerminalBlock*2pol'],do_erc=True,pins=[
            Pin(num='1',name='+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Fan_IEC60617',dest=TEMPLATE,tool=SKIDL,keywords='Fan Motor IEC-60617',description='Fan (according to IEC-60617)',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x02', 'Connect:bornier2', 'TerminalBlock*2pol'],do_erc=True,pins=[
            Pin(num='1',name='+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Fan_Tacho',dest=TEMPLATE,tool=SKIDL,keywords='Fan Motor tacho',description='Fan, tacho output, 3-pin connector',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Fan_Pin_Header_Straight_1x03', 'Pin_Headers:Pin_Header_Straight_1x03', 'TerminalBlock*3pol', 'bornier3'],do_erc=True,aliases=['Fan_3pin', 'Fan_PC_Chassis'],pins=[
            Pin(num='1',name='Tacho',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='-',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Fan_Tacho_PWM',dest=TEMPLATE,tool=SKIDL,keywords='Fan Motor tacho PWM',description='Fan, tacho output, PWM input, 4-pin connector',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Fan_Pin_Header_Straight_1x04', 'Pin_Headers:Pin_Header_Straight_1x04', 'TerminalBlock*4pol', 'bornier4'],do_erc=True,aliases=['Fan_CPU_4pin', 'Fan_4pin'],pins=[
            Pin(num='1',name='-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='Tacho',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='PWM',do_erc=True)]),
        Part(name='Motor_AC',dest=TEMPLATE,tool=SKIDL,keywords='AC Motor',description='AC Motor',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x02', 'Connect:bornier2', 'TerminalBlock*2pol'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Motor_DC',dest=TEMPLATE,tool=SKIDL,keywords='DC Motor',description='DC Motor',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x02', 'Connect:bornier2', 'TerminalBlock*2pol'],do_erc=True,pins=[
            Pin(num='1',name='+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Motor_DC_ALT',dest=TEMPLATE,tool=SKIDL,keywords='DC Motor',description='DC Motor, alternative symbol',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x02', 'Connect:bornier2', 'TerminalBlock*2pol'],do_erc=True,pins=[
            Pin(num='1',name='+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Motor_Servo',dest=TEMPLATE,tool=SKIDL,keywords='Servo Motor',description='Servo Motor (Robbe connector)',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x03'],do_erc=True,aliases=['Motor_Servo_JR', 'Motor_Servo_Hitec', 'Motor_Servo_Futaba_J', 'Motor_Servo_Robbe', 'Motor_Servo_Grapner_JR'],pins=[
            Pin(num='1',name='PWM',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='-',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Motor_Servo_AirTronics',dest=TEMPLATE,tool=SKIDL,keywords='Servo Motor',description='Servo Motor (AirTronics connector)',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x03'],do_erc=True,pins=[
            Pin(num='1',name='+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='PWM',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Stepper_Motor_bipolar',dest=TEMPLATE,tool=SKIDL,keywords='bipolar stepper motor',description='4-wire bipolar stepper motor',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x04', 'Connect:bornier4', 'TerminalBlock*4pol'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Stepper_Motor_unipolar_5pin',dest=TEMPLATE,tool=SKIDL,keywords='unipolar stepper motor',description='5-wire unipolar stepper motor',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x05', 'Connect:bornier5', 'TerminalBlock*5pol'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='Stepper_Motor_unipolar_6pin',dest=TEMPLATE,tool=SKIDL,keywords='unipolar stepper motor',description='6-wire unipolar stepper motor',ref_prefix='M',num_units=1,fplist=['Pin_Headers:Pin_Header_Straight_1x06', 'Connect:bornier6', 'TerminalBlock*6pol'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)])])


import pytest
from skidl import *
from .setup_teardown import *

def test_nets_1():
    gnd = Net('GND')
    a = Net('A')
    b = Net('B')
    c = Net()
    p = Pin()
    assert len(default_circuit.get_nets()) == 0
    assert len(a) == 0
    assert len(b) == 0
    assert len(c) == 0
    a += p
    assert len(default_circuit.get_nets()) == 1
    assert len(a) == 1
    assert len(b) == 0
    assert len(c) == 0



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

cmos4000 = SchLib(tool=SKIDL).add_parts(*[
        Part(name='14529',dest=TEMPLATE,tool=SKIDL,keywords='CMOS MUX MUX4',description='Dual 4 to 1 Multiplexer',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='STX',do_erc=True),
            Pin(num='2',name='X0',do_erc=True),
            Pin(num='3',name='X1',do_erc=True),
            Pin(num='4',name='X2',do_erc=True),
            Pin(num='5',name='X3',do_erc=True),
            Pin(num='6',name='A',do_erc=True),
            Pin(num='7',name='B',do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='Z',func=Pin.TRISTATE,do_erc=True),
            Pin(num='10',name='W',func=Pin.TRISTATE,do_erc=True),
            Pin(num='11',name='Y3',do_erc=True),
            Pin(num='12',name='Y2',do_erc=True),
            Pin(num='13',name='Y1',do_erc=True),
            Pin(num='14',name='Y0',do_erc=True),
            Pin(num='15',name='STY',do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True)]),
        Part(name='4001',dest=TEMPLATE,tool=SKIDL,keywords='CMOS Nor2',description='Quad Nor 2 inputs',ref_prefix='U',num_units=4,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4002',dest=TEMPLATE,tool=SKIDL,keywords='CMOS Nor4',description='Dual 4 input NOR gate',ref_prefix='U',num_units=2,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='1',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='40106',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4011',dest=TEMPLATE,tool=SKIDL,keywords='CMOS Nand2',description='Quad Nand 2 inputs',ref_prefix='U',num_units=4,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4012',dest=TEMPLATE,tool=SKIDL,keywords='CMOS Nand4',description='Dual 4 inputs Nand',ref_prefix='U',num_units=2,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='1',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='13',name='~',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='4013',dest=TEMPLATE,tool=SKIDL,keywords='CMOS DFF',description='Dual D FlipFlop, Set & reset',ref_prefix='U',num_units=2,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='Q',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~Q',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='Cp',do_erc=True),
            Pin(num='4',name='Cd',do_erc=True),
            Pin(num='5',name='D',do_erc=True),
            Pin(num='6',name='Sd',do_erc=True),
            Pin(num='8',name='Sd',do_erc=True),
            Pin(num='9',name='D',do_erc=True),
            Pin(num='10',name='Cd',do_erc=True),
            Pin(num='11',name='Cp',do_erc=True),
            Pin(num='12',name='~Q',do_erc=True),
            Pin(num='13',name='Q',do_erc=True)]),
        Part(name='4016',dest=TEMPLATE,tool=SKIDL,keywords='CMOS SWITCH',description='Quad Ananlog Switches',ref_prefix='U',num_units=4,do_erc=True,aliases=['4066'],pins=[
            Pin(num='7',name='V-',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='V+',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='I/O',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='O/I',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='ON',do_erc=True),
            Pin(num='3',name='O/I',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='I/O',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='ON',do_erc=True),
            Pin(num='6',name='ON',do_erc=True),
            Pin(num='8',name='I/O',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='O/I',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='O/I',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='I/O',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='ON',do_erc=True)]),
        Part(name='4017',dest=TEMPLATE,tool=SKIDL,keywords='CNT CNT10',description='Jonhson Counter ( 10 outputs )',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='Q5',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='Q1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='Q0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='Q2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='Q6',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='Q7',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='Q8',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='Q9',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='Cout',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='CKEN',do_erc=True),
            Pin(num='14',name='CLK',do_erc=True),
            Pin(num='15',name='Reset',do_erc=True)]),
        Part(name='4020',dest=TEMPLATE,tool=SKIDL,keywords='CMOS CNT CNT14',description='Binary counter 14 stages (asynchronous)',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='Q11',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='Q12',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='Q13',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='Q5',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='Q6',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='Q0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='CLK',do_erc=True),
            Pin(num='11',name='Reset',do_erc=True),
            Pin(num='12',name='Q8',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='Q7',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='Q9',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='Q10',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='4022',dest=TEMPLATE,tool=SKIDL,keywords='CMOS CNT CNT8',description='Jonhson Counter (8 states)',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='Q1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='Q0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='Q2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='Q5',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='Q6',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='Q7',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='Cout',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='CKEN',do_erc=True),
            Pin(num='14',name='CLK',do_erc=True),
            Pin(num='15',name='Reset',do_erc=True)]),
        Part(name='4023',dest=TEMPLATE,tool=SKIDL,keywords='CMOS Nand3',description='Triple Nand 3 inputs',ref_prefix='U',num_units=3,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='10',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='10',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4025',dest=TEMPLATE,tool=SKIDL,keywords='CMOS Nor3',description='Triple Nor 3 inputs',ref_prefix='U',num_units=3,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='10',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='10',name='~',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4027',dest=TEMPLATE,tool=SKIDL,keywords='CMOS JK JKFF',description='Dual JK FlipFlop, set & reset',ref_prefix='U',num_units=2,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='Q',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~Q',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='Cp',do_erc=True),
            Pin(num='4',name='Cd',do_erc=True),
            Pin(num='5',name='K',do_erc=True),
            Pin(num='6',name='J',do_erc=True),
            Pin(num='7',name='Sd',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='Sd',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='J',do_erc=True),
            Pin(num='11',name='K',do_erc=True),
            Pin(num='12',name='Cd',do_erc=True),
            Pin(num='13',name='Cp',do_erc=True),
            Pin(num='14',name='~Q',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='Q',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='4028',dest=TEMPLATE,tool=SKIDL,keywords='CMOS DECOD DECOD10',description='Decoder 4 to 10 lines',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='S4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='S2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='S0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='S7',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='S9',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='S5',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='S6',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='S8',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='A',do_erc=True),
            Pin(num='11',name='D',do_erc=True),
            Pin(num='12',name='C',do_erc=True),
            Pin(num='13',name='B',do_erc=True),
            Pin(num='14',name='S1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='S3',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='4029',dest=TEMPLATE,tool=SKIDL,keywords='CMOS COUNTER',description='Up/Down Counter',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='PE',do_erc=True),
            Pin(num='2',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='J4',do_erc=True),
            Pin(num='4',name='J1',do_erc=True),
            Pin(num='5',name='Cin',do_erc=True),
            Pin(num='6',name='Q1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Cout',do_erc=True),
            Pin(num='9',name='B/D',do_erc=True),
            Pin(num='10',name='U/D',do_erc=True),
            Pin(num='11',name='Q2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='J2',do_erc=True),
            Pin(num='13',name='J3',do_erc=True),
            Pin(num='14',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='CK',do_erc=True)]),
        Part(name='4040',dest=TEMPLATE,tool=SKIDL,keywords='CMOS CNT CNT12',description='Binary Counter 12 stages (Asynchronous)',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='Q11',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='Q5',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='Q6',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='Q2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Q1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='Q0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='CLK',do_erc=True),
            Pin(num='11',name='Reset',do_erc=True),
            Pin(num='12',name='Q8',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='Q7',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='Q9',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='Q10',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='4046',dest=TEMPLATE,tool=SKIDL,keywords='CMOS PLL',description='Phase Comp & VCO',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='PCP',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='PC1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='SigIn',do_erc=True),
            Pin(num='4',name='FOUT',do_erc=True),
            Pin(num='5',name='Inh',do_erc=True),
            Pin(num='6',name='C1',do_erc=True),
            Pin(num='7',name='C2',do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='VCOin',do_erc=True),
            Pin(num='10',name='SFout',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='R1',do_erc=True),
            Pin(num='12',name='R2',do_erc=True),
            Pin(num='13',name='PC2',func=Pin.TRISTATE,do_erc=True),
            Pin(num='14',name='RefIn',do_erc=True),
            Pin(num='15',name='ZOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True)]),
        Part(name='4049',dest=TEMPLATE,tool=SKIDL,keywords='CMOS BUFFER',description='Hex Buffer Inverter',ref_prefix='U',num_units=6,do_erc=True,pins=[
            Pin(num='1',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='~',do_erc=True),
            Pin(num='15',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='~',do_erc=True),
            Pin(num='15',name='~',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='4050',dest=TEMPLATE,tool=SKIDL,keywords='CMOS BUFFER',description='Hex Buffer',ref_prefix='U',num_units=6,do_erc=True,pins=[
            Pin(num='1',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='~',do_erc=True),
            Pin(num='15',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='~',do_erc=True),
            Pin(num='15',name='~',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='4051',dest=TEMPLATE,tool=SKIDL,keywords='CMOS MUX MUX8',description='Analog Multiplexer 8 to 1 lins',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='X4',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='X6',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='X',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='X7',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='X5',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='Inh',do_erc=True),
            Pin(num='7',name='VEE',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='C',do_erc=True),
            Pin(num='10',name='B',do_erc=True),
            Pin(num='11',name='A',do_erc=True),
            Pin(num='12',name='X3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='X0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='X1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='X2',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='4052',dest=TEMPLATE,tool=SKIDL,keywords='CMOS MUX MUX4',description='Dual Analog Multiplexer 4 to 1 line',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='Y0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='Y2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='Y',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='Y3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='Y1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='Inh',do_erc=True),
            Pin(num='7',name='VEE',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='B',do_erc=True),
            Pin(num='10',name='A',do_erc=True),
            Pin(num='11',name='X3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='X0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='X',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='X1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='X2',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='4053',dest=TEMPLATE,tool=SKIDL,keywords='CMOS MUX MUX2',description='Triple analog Multiplexer 2 to 1 line',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='Y1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='Y0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='Z1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='Z',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='Z0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='Inh',do_erc=True),
            Pin(num='7',name='VEE',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='C',do_erc=True),
            Pin(num='10',name='B',do_erc=True),
            Pin(num='11',name='A',do_erc=True),
            Pin(num='12',name='X0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='X1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='X',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='Y',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='4069',dest=TEMPLATE,tool=SKIDL,keywords='CMOD NOT',description='Hex inverter',ref_prefix='U',num_units=6,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4070',dest=TEMPLATE,tool=SKIDL,keywords='CMOS XOR2',description='Quad Xor 2 inputs',ref_prefix='U',num_units=4,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4071',dest=TEMPLATE,tool=SKIDL,keywords='CMOS OR2',description='Quad Or 2 inputs',ref_prefix='U',num_units=4,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4072',dest=TEMPLATE,tool=SKIDL,keywords='CMOS OR4',description='Dual 4 input OR gate',ref_prefix='U',num_units=2,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='1',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4073',dest=TEMPLATE,tool=SKIDL,keywords='CMOS And3',description='Triple And 3 inputs',ref_prefix='U',num_units=3,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4077',dest=TEMPLATE,tool=SKIDL,keywords='CMOS XOR XNOR XNOR2',description='Quad Xnor 2 inputs',ref_prefix='U',num_units=4,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4081',dest=TEMPLATE,tool=SKIDL,keywords='CMOS And2',description='Quad And 2 inputs',ref_prefix='U',num_units=4,do_erc=True,pins=[
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True)]),
        Part(name='4510',dest=TEMPLATE,tool=SKIDL,keywords='CMOS CNT CNT4',description='4 bits programmable counter',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='PE',do_erc=True),
            Pin(num='2',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='A4',do_erc=True),
            Pin(num='4',name='A1',do_erc=True),
            Pin(num='5',name='CI',do_erc=True),
            Pin(num='6',name='Q1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='CO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='RST',do_erc=True),
            Pin(num='10',name='U/D',do_erc=True),
            Pin(num='11',name='Q2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='A2',do_erc=True),
            Pin(num='13',name='A3',do_erc=True),
            Pin(num='14',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='CLK',do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True)]),
        Part(name='4518',dest=TEMPLATE,tool=SKIDL,keywords='CMOS',description='Dual BCD Up-Counter',ref_prefix='U',num_units=2,do_erc=True,pins=[
            Pin(num='1',name='CK',do_erc=True),
            Pin(num='2',name='Enable',do_erc=True),
            Pin(num='3',name='Q1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='Q2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Reset',do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='CK',do_erc=True),
            Pin(num='10',name='Enable',do_erc=True),
            Pin(num='11',name='Q1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='Q2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='Reset',do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True)]),
        Part(name='4520',dest=TEMPLATE,tool=SKIDL,keywords='CMOS',description='Dual Binary Up-Counter',ref_prefix='U',num_units=2,do_erc=True,pins=[
            Pin(num='1',name='CK',do_erc=True),
            Pin(num='2',name='Enable',do_erc=True),
            Pin(num='3',name='Q1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='Q2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Reset',do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='CK',do_erc=True),
            Pin(num='10',name='Enable',do_erc=True),
            Pin(num='11',name='Q1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='Q2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='Q3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='Q4',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='Reset',do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True)]),
        Part(name='4538',dest=TEMPLATE,tool=SKIDL,keywords='CMOS',description='Monostable',ref_prefix='U',num_units=2,do_erc=True,aliases=['4528', '14528', '14538'],pins=[
            Pin(num='1',name='gnd',do_erc=True),
            Pin(num='2',name='RxCx',do_erc=True),
            Pin(num='3',name='R',do_erc=True),
            Pin(num='4',name='Clk+',do_erc=True),
            Pin(num='5',name='Clk-',do_erc=True),
            Pin(num='6',name='Q',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='~Q',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='~Q',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='Q',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='Clk-',do_erc=True),
            Pin(num='12',name='Clk+',do_erc=True),
            Pin(num='13',name='R',do_erc=True),
            Pin(num='14',name='RxCx',do_erc=True),
            Pin(num='15',name='gnd',do_erc=True),
            Pin(num='16',name='VDD',func=Pin.PWRIN,do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

ttl_ieee = SchLib(tool=SKIDL).add_parts(*[
        Part(name='7400',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7401',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['7439', '7403']),
        Part(name='7402',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7404',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7405',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7406',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['7416']),
        Part(name='7407',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['7417']),
        Part(name='7408',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7409',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7410',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7411',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7412',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74126',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74426']),
        Part(name='74128',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7413',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS18']),
        Part(name='74132',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS24']),
        Part(name='74136',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7414',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS19']),
        Part(name='74141',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74147',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74148',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74151',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74153',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74154',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74155',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74156',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74157',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74158',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74159',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74164',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74165',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74166',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74173',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74176',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74196']),
        Part(name='7420',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7421',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7422',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7425',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74251',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74253',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7426',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['7438']),
        Part(name='7427',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74278',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7428',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74293',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7430',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7432',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7433',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7437',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7440',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7442',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74425',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74125']),
        Part(name='7443',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7444',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7445',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74145']),
        Part(name='7446',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS347', '7447', '74246', '74247', '74LS447']),
        Part(name='7448',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74248', '74249']),
        Part(name='7451',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7454',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7483',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7485',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7486',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7490',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7491',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7492',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7493',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7495',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='7496',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74HC237',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74HC238',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74HC36',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74HC804',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74HC805',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74HC808',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74HC832',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS133',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS137',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS138',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS139',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS15',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS152',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS161',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS160', '74LS162', '74LS163']),
        Part(name='74LS168',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS169', '74LS668', '74LS669']),
        Part(name='74LS170',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS177',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS197']),
        Part(name='74LS190',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS191']),
        Part(name='74LS192',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS193']),
        Part(name='74LS194',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS195',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS239',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS240',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS241',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS244']),
        Part(name='74LS242',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS243',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS245',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS257',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS258',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS266',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS280',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS283',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS290',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS295',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS298',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS299',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS323']),
        Part(name='74LS348',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS352',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS353',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS365',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS366',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS367',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS368',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS386',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS390',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS395',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS396',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS398',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS399',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS445',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS465',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS466',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS467',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS468',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS49',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS540',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS541',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS55',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS56',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS57']),
        Part(name='74LS590',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS591',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS594',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS595',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS596',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS597',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS599',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS620',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS640']),
        Part(name='74LS621',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS641']),
        Part(name='74LS622',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS642']),
        Part(name='74LS623',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS645']),
        Part(name='74LS638',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS639']),
        Part(name='74LS670',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS682',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS684']),
        Part(name='74LS683',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['74LS685']),
        Part(name='74LS686',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS687',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS688',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74LS689',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='74S140',dest=TEMPLATE,tool=SKIDL,do_erc=True)])